@@ -1,6 +1,12 @@
 # -*- coding: utf-8 -*-
 """Update Class file."""
 
+import dateparser
+import logging
+from requests import HTTPError
+
+from bits.github import GitHub
+
 
 class Update(object):
     """Update class."""
@@ -13,11 +19,21 @@ class Update(object):
 
         # data
         self.github_ids = {}
+        self.github_users = {}
         self.google_ids = {}
         self.people = {}
         self.role_accounts = {}
         self.users = {}
 
+    def _get_firestore_github_users(self):
+        """Return a dict of cached GitHub users from Firestore."""
+        github_users = {}
+        firestore = self.github.firestore(self.auth)
+        for doc in firestore.db.collection('github_users').stream():
+            github_id = doc.id
+            github_users[github_id] = doc.to_dict()
+        return github_users
+
     def _get_github_role_accounts(self):
         """Return a dict of github role accounts by github ID."""
         role_accounts = {}
@@ -337,3 +353,79 @@ class Update(object):
 
         if self.verbose:
             print('Done updating GitHub teams.')
+
+    def update_user(self, user):
+        """Update a single user."""
+        # check for current record
+        github_id = user['github_id']
+        login = user['login'].lower()
+        token = user['token']
+
+        # get last modified date from current record
+        last_modified = None
+        if self.github_users.get(github_id):
+            current = self.github_users[github_id]
+            updated_at = dateparser.parse(current['updated_at'])
+            last_modified = updated_at.strftime('%a, %d %b %Y %H:%M:%S GMT')
+
+        # authenticate with user's token
+        github = GitHub(token)
+
+        # get user from github api
+        try:
+            new = github.get_self(last_modified=last_modified)
+        except HTTPError as e:
+            if e.response.status_code == 401:
+                logging.warning('Invalid token for user: {} [{}]'.format(login, github_id))
+                print('Deleting user with invalid token: {} [{}]'.format(login, github_id))
+                # delete user with invalid token from user token store
+                self.github.datastore().delete_user(github_id)
+            else:
+                logging.error('Failed to get user: {} [{}]'.format(login, github_id))
+                logging.error(e)
+            return user
+
+        # don't update users that haven't changed
+        if new is False:
+            # User record has not been updated since the last time we checked.
+            if self.verbose:
+                print('Skipping {} [{}]'.format(login, github_id))
+            return user
+
+        # print(json.dumps(user, indent=2, sort_keys=True))
+        print('Updated user: {}'.format(user['login'].lower()))
+
+        # check user_id
+        user_id = str(new['id'])
+        if github_id != user_id:
+            logging.error('User ID Mismatch: {} != {}'.format(github_id, user_id))
+
+        return new
+
+    def update_users(self):
+        """Update cached data about linked users in Firestore."""
+        # get Broadies that have linked their GitHub accounts.
+        if not self.users:
+            self.users = self._get_users()
+        if self.verbose:
+            print('Found {} Tokens in GitHub App.'.format(len(self.users)))
+
+        if not self.github_users:
+            self.github_users = self._get_firestore_github_users()
+        if self.verbose:
+            print('Found {} GitHub Users in Firestore.'.format(len(self.github_users)))
+
+        # update users
+        github_users = []
+        updates = []
+        for email in sorted(self.users, key=lambda x: self.users[x]['login'].lower()):
+            user = self.users[email]
+            new = self.update_user(user)
+            github_users.append(new)
+            if new != user:
+                updates.append(new)
+
+        if self.verbose:
+            print('Updated {} github_users.'.format(len(updates)))
+
+        return github_users
